"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var types_1 = require("./types");
var decorator_util_1 = require("./utils/decorator-util");
function collectDecorators(ctor) {
    var decorators = decorator_util_1.getDecorators(ctor);
    var proto = ctor.prototype;
    proto.__states__ = [];
    proto.__getters__ = [];
    proto.__mutations__ = [];
    proto.__actions__ = [];
    decorators.forEach(function (decorator, key) {
        switch (decorator) {
            case types_1.DecoratorType.STATE:
                proto.__states__.push(key);
                break;
            case types_1.DecoratorType.GETTER:
                proto.__getters__.push(key);
                break;
            case types_1.DecoratorType.MUTATION:
                proto.__mutations__.push(key);
                break;
            case types_1.DecoratorType.ACTION:
                proto.__actions__.push(key);
                break;
        }
    });
}
exports.collectDecorators = collectDecorators;
function applyDecorators(moduleBuilder, instance) {
    var constructor = instance.constructor;
    var proto = constructor.prototype;
    proto.__states__.forEach(function (propertyName) {
        applyState(moduleBuilder, instance, propertyName);
    });
    proto.__getters__.forEach(function (propertyName) {
        applyGetter(moduleBuilder, instance, propertyName);
    });
    proto.__mutations__.forEach(function (propertyName) {
        applyMutation(moduleBuilder, instance, propertyName);
    });
    proto.__actions__.forEach(function (propertyName) {
        applyAction(moduleBuilder, instance, propertyName);
    });
}
exports.applyDecorators = applyDecorators;
function applyState(moduleBuilder, instance, propertyName) {
    var value = instance[propertyName];
    Object.defineProperty(instance, propertyName, {
        get: function () {
            return moduleBuilder.state()[propertyName];
        },
        set: function (val) {
            moduleBuilder.state()[propertyName] = val;
        }
    });
    instance[propertyName] = value;
}
function applyGetter(moduleBuilder, instance, propertyName) {
    var getterFunction = instance.__lookupGetter__(propertyName) || (function (state) { return ({}); });
    getterFunction = getterFunction.bind(instance);
    var getter = function (state) {
        return getterFunction();
    };
    moduleBuilder.addGetter(propertyName, getter);
    Object.defineProperty(instance, propertyName, {
        get: function () {
            return moduleBuilder.read(propertyName);
        }
    });
}
function applyMutation(moduleBuilder, instance, propertyName) {
    var mutationFunction = instance[propertyName] || (function (payload) { return undefined; });
    mutationFunction = mutationFunction.bind(instance);
    var mutation = function (state, payload) {
        mutationFunction(payload);
    };
    moduleBuilder.addMutation(propertyName, mutation);
    instance[propertyName] = function (payload) {
        moduleBuilder.commit(propertyName, payload);
    };
}
function applyAction(moduleBuilder, instance, propertyName) {
    var actionFunction = instance[propertyName] || (function (payload) { return Promise.resolve(); });
    actionFunction = actionFunction.bind(instance);
    var action = function (context, payload) {
        return actionFunction(payload);
    };
    moduleBuilder.addAction(propertyName, action);
    instance[propertyName] = function (payload) {
        return moduleBuilder.dispatch(propertyName, payload);
    };
}
//# sourceMappingURL=metadata.js.map