"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var ModuleBuilder = /** @class */ (function () {
    function ModuleBuilder(namespace, state) {
        this._namespace = namespace;
        this._module = {
            actions: {},
            getters: {},
            mutations: {},
            namespaced: true,
            state: state
        };
    }
    ModuleBuilder.prototype.setStoreBuilder = function (storeBuilder) {
        this._storeBuilder = storeBuilder;
    };
    ModuleBuilder.prototype.addAction = function (name, action) {
        if (this._module.actions) {
            this._module.actions[name] = action;
        }
    };
    ModuleBuilder.prototype.addMutation = function (name, mutation) {
        if (this._module.mutations) {
            this._module.mutations[name] = mutation;
        }
    };
    ModuleBuilder.prototype.addGetter = function (name, getter) {
        if (this._module.getters) {
            this._module.getters[name] = getter;
        }
    };
    ModuleBuilder.prototype.state = function () {
        return this._storeBuilder && this._storeBuilder.store
            ? this._storeBuilder.store.state[this._namespace]
            : this._module.state;
    };
    ModuleBuilder.prototype.commit = function (name, payload) {
        if (this._storeBuilder && this._storeBuilder.store) {
            return this._storeBuilder.store.commit(this.namespace + '/' + name, payload);
        }
        else {
            throw new Error('Could not commit: no store created');
        }
    };
    ModuleBuilder.prototype.read = function (name) {
        if (this._storeBuilder && this._storeBuilder.store) {
            return this._storeBuilder.store.getters[this.namespace + '/' + name];
        }
        else {
            throw new Error('Could not commit: no store created');
        }
    };
    ModuleBuilder.prototype.dispatch = function (name, payload) {
        if (this._storeBuilder && this._storeBuilder.store) {
            return this._storeBuilder.store.dispatch(this.namespace + '/' + name, payload);
        }
        else {
            throw new Error('Could not dispatch: no store created');
        }
    };
    Object.defineProperty(ModuleBuilder.prototype, "module", {
        get: function () {
            return this._module;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(ModuleBuilder.prototype, "namespace", {
        get: function () {
            return this._namespace;
        },
        enumerable: true,
        configurable: true
    });
    return ModuleBuilder;
}());
exports.ModuleBuilder = ModuleBuilder;
//# sourceMappingURL=module-builder.js.map