"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
var typedi_1 = require("typedi");
var vuex_1 = require("vuex");
var StoreBuilder = /** @class */ (function () {
    function StoreBuilder(options) {
        if (options === void 0) { options = {}; }
        this.initialize(options);
    }
    Object.defineProperty(StoreBuilder.prototype, "container", {
        get: function () {
            return this._container;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(StoreBuilder.prototype, "store", {
        get: function () {
            return this._store;
        },
        enumerable: true,
        configurable: true
    });
    StoreBuilder.prototype.initialize = function (options) {
        this._container = options.container || typedi_1.Container.of(undefined);
        this._options = {
            actions: options.actions || {},
            getters: options.getters || {},
            modules: options.modules || {},
            mutations: options.mutations || {},
            plugins: options.plugins || [],
            state: options.state,
            strict: options.strict || true
        };
    };
    StoreBuilder.prototype.useContainer = function (container) {
        this._container = container;
    };
    StoreBuilder.prototype.setStrict = function (strict) {
        this._options.strict = strict;
    };
    StoreBuilder.prototype.merge = function (options) {
        this._options = {
            actions: tslib_1.__assign({}, this._options.actions, (options.actions || {})),
            getters: tslib_1.__assign({}, this._options.getters, (options.getters || {})),
            modules: tslib_1.__assign({}, this._options.modules, (options.modules || {})),
            mutations: tslib_1.__assign({}, this._options.mutations, (options.mutations || {})),
            plugins: this._options.plugins.concat((options.plugins || [])),
            state: tslib_1.__assign({}, this._options.state, (options.state || {})),
            strict: options.strict || this._options.strict
        };
    };
    StoreBuilder.prototype.loadModules = function (modules) {
        var _this = this;
        modules.forEach(function (moduleClass) {
            _this.loadModule(moduleClass);
        });
    };
    StoreBuilder.prototype.loadModule = function (moduleClass) {
        var instance = this._container.get(moduleClass);
        if (!instance) {
            instance = new moduleClass();
            this._container.set(moduleClass, instance);
        }
        var moduleBuilder = instance.__moduleBuilder__;
        if (this._options.modules) {
            moduleBuilder.setStoreBuilder(this);
            this._options.modules[moduleBuilder.namespace] = moduleBuilder.module;
        }
    };
    StoreBuilder.prototype.addModule = function (namespace, moduleOptions) {
        if (this._options.modules) {
            this._options.modules[namespace] = moduleOptions;
        }
    };
    StoreBuilder.prototype.create = function () {
        this._store = new vuex_1.Store(this._options);
        this._store.__storeBuilder__ = this;
        return this._store;
    };
    return StoreBuilder;
}());
exports.StoreBuilder = StoreBuilder;
var storeBuilderSingleton = new StoreBuilder({
    state: {},
    strict: true
});
var namedStoreBuilderMap = {};
function getStoreBuilder(name) {
    // the default store builder
    if (!name) {
        return storeBuilderSingleton;
    }
    // a named store builder
    var builder = namedStoreBuilderMap[name] ||
        (namedStoreBuilderMap[name] = new StoreBuilder({
            state: {},
            strict: true
        }));
    return builder;
}
exports.getStoreBuilder = getStoreBuilder;
//# sourceMappingURL=store-builder.js.map