"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
var typedi_1 = require("typedi");
var vue_1 = tslib_1.__importDefault(require("vue"));
var vue_class_component_1 = require("vue-class-component");
var KEY = '__injections__';
/**
 * Thrown when DI cannot inject value into property decorated by @Inject decorator.
 */
var CannotInjectError = /** @class */ (function (_super) {
    tslib_1.__extends(CannotInjectError, _super);
    function CannotInjectError(target, propertyName) {
        var _this = _super.call(this, "Cannot inject value into \"" + target.constructor.name + "." + propertyName + "\". " +
            "Please make sure you setup reflect-metadata properly and you don't use interfaces without service tokens as injection value.") || this;
        _this.name = 'CannotInjectError';
        Object.setPrototypeOf(_this, CannotInjectError.prototype);
        return _this;
    }
    return CannotInjectError;
}(Error));
exports.CannotInjectError = CannotInjectError;
function registerInjection(target, propertyName, typeOrName, index) {
    if (target instanceof vue_1.default) {
        var decorator = vue_class_component_1.createDecorator(function (options) {
            addInjection(target, options, propertyName, typeOrName, index);
        });
        decorator(target, propertyName, index);
        return;
    }
    // still support injection outside of vue components for backward compatibility
    addInjection(target, target.constructor, propertyName, typeOrName, index);
}
exports.registerInjection = registerInjection;
/**
 * Inject all properties marked with @Inject() for the given class instance
 * @param instance Class instance
 */
function injectAll(instance, container) {
    var _this = this;
    var target = instance instanceof vue_1.default ? instance.$options : instance.constructor;
    var injections = getInjections(target);
    if (injections) {
        injections.forEach(function (injection) {
            var typeOrName = injection.typeOrName, propertyName = injection.propertyName;
            var identifier;
            if (typeof typeOrName === 'string') {
                identifier = typeOrName;
            }
            else if (typeOrName instanceof typedi_1.Token) {
                identifier = typeOrName;
            }
            else if (typeOrName) {
                identifier = typeOrName();
            }
            if (identifier === Object) {
                throw new CannotInjectError(_this, propertyName);
            }
            if (container && container.has(identifier)) {
                instance[propertyName] = container.get(identifier);
            }
            else {
                instance[propertyName] = typedi_1.Container.get(identifier);
            }
        });
    }
}
exports.injectAll = injectAll;
function initializeInjections(ctor) {
    return (ctor[KEY] = []);
}
function getInjections(ctor) {
    return ctor[KEY];
}
function addInjection(target, ctor, propertyName, typeOrName, index) {
    var injections = getInjections(ctor);
    if (!injections) {
        injections = initializeInjections(ctor);
    }
    if (!typeOrName) {
        typeOrName = function () { return Reflect.getMetadata('design:type', target, propertyName); };
    }
    injections.push({
        index: index,
        propertyName: propertyName,
        typeOrName: typeOrName
    });
}
//# sourceMappingURL=inject-util.js.map